#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Des 17 06:13:10 2021

@author: hakimbmkg
"""

import os
import json
from tqdm import tqdm
import pandas as pd
from obspy import UTCDateTime
from obspy.clients.fdsn.client import Client

class Data:
    """
    Class for preprocessing data
    """
    global directory

    directory = os.getcwd()
    
    def make_station(net, dayStart, dayEnd, min_lat, max_lat, min_lng, max_lng, url,user=None,pawd=None):
        """
        Function for make station list
        """

        url_ = str(url)
        usr_ = str(user)
        pwd_ = str(pawd)
        client = Client(url_,user=usr_,password=pwd_) 

        if not os.path.exists(directory+'/input/station/'):
            os.makedirs(directory+'/input/station')
            print('== folder /input/station/ created')

        if os.path.exists(directory+'/input/station/stations.json'):
            os.remove(directory+'/input/station/stations.json')

        start_time  = UTCDateTime(dayStart)
        end_time    = UTCDateTime(dayEnd)

        inv = client.get_stations(
            network=net, level='channel',
            starttime=start_time, endtime=end_time,
            minlatitude=min_lat, maxlatitude=max_lat,
            minlongitude=min_lng, maxlongitude=max_lng)

        stations = {}
        for inv_ in tqdm(inv):
            network = inv_.code
            for sta in inv_:
                station          = sta.code
                latitude_        = sta.latitude
                longitude_       = sta.longitude
                elvation_        = sta.elevation
                stations[str(station)] = {
                    "network"   : network,
                    "channels"  : ['BHE','BHN','BHZ','SHE','SHN','SHZ'],
                    "coords"    : [latitude_, longitude_, elvation_]
                }
        
        folder_path = directory+'/input/station/'
        with open(folder_path+'stations.json', 'w') as file_:
            json.dump(stations, file_)
        print(f'process completed, stations.json Created')

    def download_waveform_fromcsv(path, url,user=None,pawd=None):
        """
        function for download waveform from csv file
        """

        url_ = str(url)
        usr_ = str(user)
        pwd_ = str(pawd)
        client = Client(url_,user=usr_,password=pwd_) 

        if not os.path.exists(directory+'/input/waveform/'):
            os.makedirs(directory+'/input/waveform')
            print('== folder /input/waveform/ created')
        
        if not os.path.exists(directory+'/input/labels_csv/'):
            os.makedirs(directory+'/input/labels_csv')
            print('== folder /input/labels_csv/ created')

        if not os.path.exists(directory+'/input/tmp_files/'):
            os.makedirs(directory+'/input/tmp_files')
            print('== folder /input/tmp_files/ created')

        if os.path.exists(directory+'/input/tmp_files/tmp_inaseisform.csv'):
            os.remove(directory+'/input/tmp_files/tmp_inaseisform.csv')

        read_csv = pd.read_csv(directory+'/'+path)
        init_stations = read_csv[['start_time','end_time','network','code','classID','class']]
        arr_init_stations = init_stations.to_numpy()
        xx = len(arr_init_stations)

        df = pd.DataFrame(columns=[
            'files_name',
            'start_time',
            'end_time',
            'network',
            'stations_code',
            'classID',
            'class'
        ])
        df.to_csv(directory+'/input/tmp_files/tmp_inaseisform.csv', mode='a', header=True, index=False)
        
        for num_,i in enumerate(tqdm(arr_init_stations, desc='Process')):
            xx = str(num_)
            try:
                time_   = (str(i[1].replace('-',''))+str(i[0].replace('-','')))
                t1  = UTCDateTime(i[0])
                t2  = UTCDateTime(i[1])
                st  = client.get_waveforms(i[2],i[3],'*','BH?',t1,t2)
                fn  = str(i[2]+'.'+i[3]+'.'+time_+xx)
                # print(fn)
                # print(st)
            except:
                try:
                    time_   = (str(i[1].replace('-',''))+str(i[0].replace('-','')))
                    t1  = UTCDateTime(i[0])
                    t2  = UTCDateTime(i[1])
                    st  = client.get_waveforms(i[2],i[3],'*','SH?',t1,t2)
                    fn  = str(i[2]+'.'+i[3]+'.'+time_+xx)
                    # print(fn)
                    # print(st)
                except:
                    st = 'NODATA'
                    print(f'***!warning!*** >> NoDATA for station {i[3]}')
                    print(f'*******************************************')

            if st == 'NODATA':
                print(f'***!warning!*** >> NoDATA for station {i[3]}')
                print(f'===========================================')
            else:
                print(f'\nData Available for station {i[3]}')
                savefile_path = str(directory+'/input/waveform/'+fn)
                try:
                    if not os.path.exists(directory+'/input/waveform/'+fn):
                        st.write(savefile_path, format='MSEED')
                        print(f'Success write mseed file for station {i[3]}')
                        print(f'===========================================')
                    else:
                        print(f'file mseed for station {i[3]} is exist')
                        print(f'===========================================')

                    data_ = {
                        'files_name'        : [fn],
                        'start_time'        : [t1],
                        'end_time'          : [t2],
                        'network'           : [i[2]],
                        'stations_code'     : [i[3]],
                        'classID'           : [i[4]],
                        'class'             : [i[5]]
                    }
                    df = pd.DataFrame(data_)
                    df.to_csv(directory+'/input/tmp_files/tmp_inaseisform.csv', mode='a', header=False, index=False)

                except:
                    print(f'***!warning!*** >> cant write mseed for station {i[3]}')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Jan  02 06:24:03 2022

@author: hakimbmkg
"""

from obspy.clients.fdsn.client import Client
from obspy.core.utcdatetime import UTCDateTime
from obspy import read
import pandas as pd
import numpy as np
import json
import os
from tqdm import tqdm
import concurrent.futures


class Eqconvert:
    """
    This is class for convert data from arrival format seiscomp and next step download by event from FDSN 
    """

    def created_station(net, dayStart, dayEnd, min_lat, max_lat, min_lng, max_lng, url,user=None,pawd=None):
        """
        Created Station Using Selected Region From FDSN
        This param is : \n
        net         : [str] network of station
        sta         : [str] name of station
        dayStart    : [day format - YYYY-MM-DDTHH:MM:SS] start time
        dayEnd      : [day format - YYYY-MM-DDTHH:MM:SS] end time
        min_lat     : [float] - min of latitude
        max_lat     : [float] - max of latitude
        min_lng     : [float] - min of longitude
        max_lng     : [float] - max of longitude
        url         : [str] url of FDSN
        user        : [str] default None
        pawd        : [str] default None

        """
        fd = os.getcwd()
        if not os.path.exists(fd+'/input/dataset_EQ/rawdata/'): 
            os.makedirs(fd+'/input/dataset_EQ/rawdata/')
            print('folder created')
        else:
            print('folder is exist')

        if not os.path.exists(fd+'/input/dataset_EQ/rawdata/station/'):
            os.makedirs(fd+'/input/dataset_EQ/rawdata/station/')
            print('folder created')
        else:
            print('folder is exist')

        url_ = str(url)
        usr_ = str(user)
        pwd_ = str(pawd)
        c = Client(url_,user=usr_,password=pwd_) 
        start_t = UTCDateTime(dayStart)
        end_t = UTCDateTime(dayEnd)

        inv = c.get_stations(
            network=net, level='channel',
            starttime=start_t, endtime=end_t,
            minlatitude=min_lat, maxlatitude=max_lat,
            minlongitude=min_lng, maxlongitude=max_lng)

        stations = {}
        for inv_ in inv:
            network = inv_.code
            for sta in inv_:
                station          = sta.code
                latitude_        = sta.latitude
                longitude_       = sta.longitude
                elvation_        = sta.elevation
                # channels_        = sta.channels[:1][0]
                # print (f'{str(network)} - {str(station)} ')
                stations[str(station)] = {
                    "network"   : network,
                    "channels"  : ['BHE','BHN','BHZ'],
                    "coords"    : [latitude_, longitude_, elvation_]
                 }
        fd_ = fd+'/input/dataset_EQ/rawdata/station/'
        with open(fd_+'stations.json', 'w') as file_:
            json.dump(stations, file_)

    def arrivalformat2csv (file_inp):

        """
        This is a function for reformat from arrival format seiscomp to CSV 
        Important note: the function has a bug, the result of reformat duplicate stations if have P and S time phase. 
        so, the solution you can remove duplicates using pandas module to remove duplicates. \n

        header of DataFrame/CSV reffer from STEAD, please follow :
        Mousavi, S. M., Sheng, Y., Zhu, W., Beroza G.C., (2019). 
        STanford EArthquake Dataset (STEAD): A Global Data Set of Seismic Signals for AI, IEEE Access, 
        doi:10.1109/ACCESS.2019.2947848
        https://github.com/smousavi05/STEAD
        """

        path_fd = os.getcwd()

        if not os.path.exists(path_fd+"/input/"):
            os.makedirs(path_fd+'/input')
            print('folder input created')

        if not os.path.exists(path_fd+"/input/dataset_EQ/"):
            os.makedirs(path_fd+"/input/dataset_EQ/")
            print('folder dataset_EQ created')
            
        if os.path.exists(path_fd+"/input/dataset_EQ/merge.csv"):
            os.remove(path_fd+"/input/dataset_EQ/merge.csv")

        df = pd.DataFrame(columns= [
            'network_code',
            'receiver_code',
            'receiver_type',
            'receiver_latitude',
            'receiver_longitude',
            'receiver_elevation_m',
            'p_arrival_sample',
            'p_status',
            'p_weight',
            'p_travel_sec',
            's_arrival_sample',
            's_status',
            's_weight',
            'source_id',
            'date_',
            'source_origin_time',
            'source_origin_uncertainty_sec',
            'source_latitude',
            'source_longitude',
            'source_error_sec',
            'source_gap_deg',
            'source_horizontal_uncertainty_km',
            'source_depth_km',
            'source_depth_uncertainty_km',
            'source_magnitude',
            'source_magnitude_type',
            'source_magnitude_author',
            'source_mechanism_strike_dip_rake',
            'source_distance_deg',
            'source_distance_km',
            'back_azimuth_deg',
            'snr_db','coda_end_sample',
            'trace_start_time',
            'trace_category',
            'trace_name'])

        df.to_csv(path_fd+'/input/dataset_EQ/merge.csv',mode='a', header=True, index=False)


        fd = open(path_fd+'/'+file_inp,'r')
        l_fd = fd.readlines()
        for i in range(len(l_fd)):
            l_fd[i] = l_fd[i].split()

        pbar = tqdm(desc='convert arr2csv', total=len(l_fd))
        i = 0
        while i < len(l_fd):
            # time.sleep(0.1)
            if len(l_fd[i])>2 and l_fd[i][2][:3] == 'bmg':
                idEvent = l_fd[i][2]

            if len(l_fd[i])>0 and l_fd[i][0] == 'Origin:':
                datex   = l_fd[i+2][1]
                timex   = l_fd[i+3][1]
                lat     = l_fd[i+4][1]
                lng     = l_fd[i+5][1]
                depth   = l_fd[i+6][1]
                auth    = l_fd[i+7][1]
                mode_   = l_fd[i+9][1]
                azgap   = l_fd[i+13][2]
                date_   = (str(datex.replace('-','')))
                time_   = (str(timex.replace(':','')))

            if len(l_fd[i])>3 and l_fd[i][-2]=='preferred':
                m     = l_fd[i][1]
                m_typ = l_fd[i][0]

            if len(l_fd[i])>4 and l_fd[i][4]=='phase':
                try:
                    k=0
                    while k<1:
                        i += 1
                        try:
                            net  = l_fd[i][1]
                            stas = l_fd[i][0]
                            phase= l_fd[i][4]
                            p_tim = ''
                            s_tim = ''
                            if phase == 'P' :    
                                p_tim= l_fd[i][5]

                                if l_fd[i+1][4]  == 'S':
                                    s_tim= l_fd[i+1][5]

                            pick_p = p_tim
                            pick_s = s_tim
                            # get detail from json station
                            station_file = open(path_fd+'/input/dataset_EQ/rawdata/station/stations.json')
                            station_load = json.load(station_file)
                            for sta in station_load:
                                sta_name = sta
                                if sta_name == stas:
                                    lat_sta = station_load[str(sta)]['coords'][0]
                                    lng_sta = station_load[str(sta)]['coords'][1]
                                    elev_sta= station_load[str(sta)]['coords'][2] 

                            lt_st = lat_sta
                            lg_st = lng_sta
                            el_st = elev_sta

                            data_ = {
                                'network_code'                  :   [net],
                                'receiver_code'                 :   [stas],
                                'receiver_type'                 :   ['BHZ'],
                                'receiver_latitude'             :   [lt_st],
                                'receiver_longitude'            :   [lg_st],
                                'receiver_elevation_m'          :   [el_st],
                                'p_arrival_sample'              :   [pick_p],
                                'p_status'                      :   [mode_],
                                'p_weight'                      :   ['None'],
                                'p_travel_sec'                  :   ['None'],
                                's_arrival_sample'              :   [pick_s],
                                's_status'                      :   [mode_],
                                's_weight'                      :   ['None'],
                                'source_id'                     :   [idEvent],
                                'date_'                         :   [datex],
                                'source_origin_time'            :   [datex +' '+timex],
                                'source_origin_uncertainty_sec' :   ['None'],
                                'source_latitude'               :   [lat],
                                'source_longitude'              :   [lng],
                                'source_error_sec'              :   ['None'],
                                'source_gap_deg'                :   [azgap],
                                'source_horizontal_uncertainty_km': ['None'],
                                'source_depth_km'               :   [depth],
                                'source_depth_uncertainty_km'   :   ['None'],
                                'source_magnitude'              :   [m],
                                'source_magnitude_type'         :   [m_typ],
                                'source_magnitude_author'       :   [auth],
                                'source_mechanism_strike_dip_rake': ['None'],
                                'source_distance_deg'           :   ['None'],
                                'source_distance_km'            :   ['None'],
                                'back_azimuth_deg'              :   ['None'],
                                'snr_db'                        :   ['None'],
                                'coda_end_sample'               :   ['None'],
                                'trace_start_time'              :   ['None'],
                                'trace_category'                :   ['None'],
                                'trace_name'                    :   [stas+'.'+net+'_'+date_+time_+'_EV']
                            }
                            df = pd.DataFrame(data_)
                            df.to_csv(path_fd+'/input/dataset_EQ/merge.csv',mode='a', header=False, index=False)

                            pbar.update()
                        except:
                            break
                except:
                    break
            i += 1
    
        Eqconvert.csvcleaning('/input/dataset_EQ/merge.csv')

    def csvcleaning(csv_path):
        path_fd = os.getcwd()
        if not os.path.exists(path_fd+"/input/dataset_EQ"):
            os.makedirs(path_fd+'/input/dataset_EQ')
        if not os.path.exists(path_fd+"/input/dataset_EQ/event"):
            os.makedirs(path_fd+'/input/dataset_EQ/event')
        if not os.path.exists(path_fd+"/input/dataset_EQ/noise"):
            os.makedirs(path_fd+'/input/dataset_EQ/noise')
            print('folder dataset created')

        if os.path.exists(path_fd+"/input/dataset_EQ/merge_clear.csv"):
            os.remove(path_fd+"/input/dataset_EQ/merge_clear.csv")
        path_fd = os.getcwd()
        df = pd.read_csv(path_fd+'/'+csv_path)
        df.dropna(subset=['p_arrival_sample'],inplace=True)
        df.to_csv(path_fd+'/input/dataset_EQ/merge_clear.csv',mode='a', header=True, index=False)
        print (df)

    def downloadseedbycsv(csv_path, url,user=None, pawd=None, n_cpu = os.cpu_count()):
        """
        function for download mseed from FDSN
        This param is : \n
        csv_path    : /your/path/csv
        url         : [str] url of FDSN
        user        : [str] default None
        pawd        : [str] default None
        """
        url_ = str(url)
        usr_ = str(user)
        pwd_ = str(pawd)
        c = Client(url_,user=usr_,password=pwd_) 

        path_fd = os.getcwd()
        df = pd.read_csv(path_fd+'/'+csv_path)
        a = df[['network_code','receiver_code','p_arrival_sample','date_','trace_name']]
        
        #parallel init
        split_df = np.array_split(a,n_cpu)
        df_results = []
        
        def work(x):
            df = x
            arr_df = df.to_numpy()
            for i in arr_df:
                try:
                    t1 = UTCDateTime(i[3]+'T'+i[2])
                    t2 = t1 + 180
                    st = c.get_waveforms(i[0],i[1],'00','BHZ',t1,t2)
                    fn = i[4]
                    # print(st)
                except:
                    try:
                        t1 = UTCDateTime(i[3]+'T'+i[2])
                        t2 = t1 + 180
                        st = c.get_waveforms(i[0],i[1],'01','BHZ',t1,t2)
                        fn = i[4]
                        # print(st)
                    except:
                        try:
                            t1 = UTCDateTime(i[3]+'T'+i[2])
                            t2 = t1 + 180
                            st = c.get_waveforms(i[0],i[1],'*','BHZ',t1,t2)
                            fn = i[4]
                            # print(st)
                        except:
                            print(f'***!warning!*** >> station '+i[1]+' - '+ i[4]+' is null')
                            st = 'null'
                            df.drop(df.loc[df['trace_name']==i[4]].index, inplace=True)
                            #df.to_csv(path_fd+'/input/dataset_EQ/x_merge_stream.csv', header=True, index=False)
                            # print (df)

                if st != 'null':
                    file_path = str(path_fd+'/input/dataset_EQ/event/'+fn)
                    try:
                        if not os.path.isfile(file_path):
                            st.write(file_path, format='MSEED')
                            print(f'==> mseed write for station '+ i[0] +'_'+ i[1])
                    except:
                        print(f'## error write file ##')
            return(df)

        #parallel run
        with concurrent.futures.ThreadPoolExecutor(max_workers=n_cpu) as executor:
            results = [ executor.submit(work,x=df) for df in split_df ]
            for z in concurrent.futures.as_completed(results):
                try:
                    df_results.append(z.result())
                except Exception as ex:
                    print(str(ex))
                    pass
        
        #output processing
        df_results = pd.concat(df_results)
        df_results['marker'] = 1
        joined = pd.merge(df,df_results, on = ['network_code','receiver_code','p_arrival_sample','date_','trace_name'], how='left')
        joined = joined[~pd.isnull(joined['marker'])]
        joined.drop(labels = 'marker', axis = 1,inplace=True)
        joined.to_csv(path_fd+'/input/dataset_EQ/merge_stream.csv', header=True, index=False)
        print('***download finish***')
        
    def checkwaveform(path_wave):
        st = read(path_wave)
        st.plot()


    



from LinduAI.main.model import Models
from LinduAI.main.modelmag import Modelsmag
from LinduAI.preprocessing.data import Data
from LinduAI.preprocessing.transform import Transform
from LinduAI.main.datas import Datas
from LinduAI.utils.eqconvert import Eqconvert


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Dec 18 06:01:03 2021

@author: hakimbmkg
"""

import os
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'

import pandas as pd
import librosa
import librosa.display
import matplotlib.pyplot as plt
plt.rcParams['agg.path.chunksize'] = 1000000
from tqdm import tqdm
import numpy as np
from obspy import read
from obspy import UTCDateTime
import tensorflow as tf
import shutil

class HParams(object):
    """ 
    
    """
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

class Transform:
    """
    Class for preprocessing data step two, transform data waveform.
    """

    global directory
    directory = os.getcwd()

    def conditioning_waveform(path):
        """
        funtion for conditioning 3 channels, BHZ, BHN, BHE, SHZ, SHN, SHE
        resample to 100Hz for standardization all mseed
        """
        
        if os.path.exists(directory+'/input/labels_csv/inaseisform.csv'):
            os.remove(directory+'/input/labels_csv/inaseisform.csv')

        read_csv = pd.read_csv(directory+'/'+path)
        init_stations = read_csv[['files_name','start_time','end_time','network','stations_code','classID','class']]
        arr_init_stations = init_stations.to_numpy()

        df = pd.DataFrame(columns=[
            'files_name',
            'start_time',
            'end_time',
            'network',
            'stations_code',
            'classID',
            'class'
        ])
        df.to_csv(directory+'/input/labels_csv/inaseisform.csv', mode='a', header=True, index=False)
        
        for n,i in enumerate(tqdm(arr_init_stations, desc='Conditioning/Split Process')):
            st = read(directory+'/input/waveform/'+i[0])
            st_net  = st[0].stats['network']
            st_code = st[0].stats['station']
            st_loc  = st[0].stats['location']
            st_fn   = st_net+'.'+st_code+'.'+st_loc+'.'

            if st.select(id=st_fn+'BHN'):
                new_st = st.select(id=st_fn+'BHN').resample(100.0)
                # print(f'resample to 100Hz for station {i[4]}')
                # print(new_st)
                new_fnN = st_fn+'BHN.'+str(UTCDateTime(i[1]))+'__'+str(UTCDateTime(i[2]))
                data_ = {
                    'files_name'        : new_fnN,
                    'start_time'        : str(UTCDateTime(i[1])),
                    'end_time'          : str(UTCDateTime(i[2])),
                    'network'           : [i[3]],
                    'stations_code'     : [i[4]],
                    'classID'           : [i[5]],
                    'class'             : [i[6]]
                    }
                df = pd.DataFrame(data_)
                df.to_csv(directory+'/input/labels_csv/inaseisform.csv', mode='a', header=False, index=False)
                try:
                    if not os.path.exists(directory+'/input/waveform/'+new_fnN):
                        new_st.write(directory+'/input/waveform/'+new_fnN, format='MSEED')
                        print(f'Success write mseed file for station {i[4]}')
                        print(f'===========================================')
                    else:
                        print(f'file mseed for station {i[4]} is exist')
                        print(f'===========================================')
                except:
                    print(f'***!warning!*** >> cant write mseed for station {i[4]}')


            if st.select(id=st_fn+'BHE'):
                new_st = st.select(id=st_fn+'BHE').resample(100.0)
                new_fnE = st_fn+'BHE.'+str(UTCDateTime(i[1]))+'__'+str(UTCDateTime(i[2]))  
                data_ = {
                    'files_name'        : new_fnE,
                    'start_time'        : str(UTCDateTime(i[1])),
                    'end_time'          : str(UTCDateTime(i[2])),
                    'network'           : [i[3]],
                    'stations_code'     : [i[4]],
                    'classID'           : [i[5]],
                    'class'             : [i[6]]
                    }
                df = pd.DataFrame(data_)
                df.to_csv(directory+'/input/labels_csv/inaseisform.csv', mode='a', header=False, index=False)
                try:
                    if not os.path.exists(directory+'/input/waveform/'+new_fnE):
                        new_st.write(directory+'/input/waveform/'+new_fnE, format='MSEED')
                        print(f'Success write mseed file for station {i[4]}')
                        print(f'===========================================')
                    else:
                        print(f'file mseed for station {i[4]} is exist')
                        print(f'===========================================')
                except:
                    print(f'***!warning!*** >> cant write mseed for station {i[4]}')



            if st.select(id=st_fn+'BHZ'):
                new_st = st.select(id=st_fn+'BHZ').resample(100.0)
                new_fnZ = st_fn+'BHZ.'+str(UTCDateTime(i[1]))+'__'+str(UTCDateTime(i[2]))
                data_ = {
                    'files_name'        : new_fnZ,
                    'start_time'        : str(UTCDateTime(i[1])),
                    'end_time'          : str(UTCDateTime(i[2])),
                    'network'           : [i[3]],
                    'stations_code'     : [i[4]],
                    'classID'           : [i[5]],
                    'class'             : [i[6]]
                    }
                df = pd.DataFrame(data_)
                df.to_csv(directory+'/input/labels_csv/inaseisform.csv', mode='a', header=False, index=False)
                try:
                    if not os.path.exists(directory+'/input/waveform/'+new_fnZ):
                        new_st.write(directory+'/input/waveform/'+new_fnZ, format='MSEED')
                        print(f'Success write mseed file for station {i[4]}')
                        print(f'===========================================')
                    else:
                        print(f'file mseed for station {i[4]} is exist')
                        print(f'===========================================')
                except:
                    print(f'***!warning!*** >> cant write mseed for station {i[4]}')

                
            if st.select(id=st_fn+'SHN'):
                new_st = st.select(id=st_fn+'SHN').resample(100.0)
                new_fnSN = st_fn+'SHN.'+str(UTCDateTime(i[1]))+'__'+str(UTCDateTime(i[2]))
                data_ = {
                    'files_name'        : new_fnSN,
                    'start_time'        : str(UTCDateTime(i[1])),
                    'end_time'          : str(UTCDateTime(i[2])),
                    'network'           : [i[3]],
                    'stations_code'     : [i[4]],
                    'classID'           : [i[5]],
                    'class'             : [i[6]]
                    }
                df = pd.DataFrame(data_)
                df.to_csv(directory+'/input/labels_csv/inaseisform.csv', mode='a', header=False, index=False)
                try:
                    if not os.path.exists(directory+'/input/waveform/'+new_fnSN):
                        new_st.write(directory+'/input/waveform/'+new_fnSN, format='MSEED')
                        print(f'Success write mseed file for station {i[4]}')
                        print(f'===========================================')
                    else:
                        print(f'file mseed for station {i[4]} is exist')
                        print(f'===========================================')
                except:
                    print(f'***!warning!*** >> cant write mseed for station {i[4]}')

                
            if st.select(id=st_fn+'SHE'):
                new_st = st.select(id=st_fn+'SHE').resample(100.0)
                new_fnSE = st_fn+'SHE.'+str(UTCDateTime(i[1]))+'__'+str(UTCDateTime(i[2]))
                data_ = {
                    'files_name'        : new_fnSE,
                    'start_time'        : str(UTCDateTime(i[1])),
                    'end_time'          : str(UTCDateTime(i[2])),
                    'network'           : [i[3]],
                    'stations_code'     : [i[4]],
                    'classID'           : [i[5]],
                    'class'             : [i[6]]
                    }
                df = pd.DataFrame(data_)
                df.to_csv(directory+'/input/labels_csv/inaseisform.csv', mode='a', header=False, index=False)
                try:
                    if not os.path.exists(directory+'/input/waveform/'+new_fnSE):
                        new_st.write(directory+'/input/waveform/'+new_fnSE, format='MSEED')
                        print(f'Success write mseed file for station {i[4]}')
                        print(f'===========================================')
                    else:
                        print(f'file mseed for station {i[4]} is exist')
                        print(f'===========================================')
                except:
                    print(f'***!warning!*** >> cant write mseed for station {i[4]}')
                

            if st.select(id=st_fn+'SHZ'):
                new_st = st.select(id=st_fn+'SHZ').resample(100.0)
                new_fnSZ = st_fn+'SHZ.'+str(UTCDateTime(i[1]))+'__'+str(UTCDateTime(i[2]))

                data_ = {
                    'files_name'        : new_fnSZ,
                    'start_time'        : str(UTCDateTime(i[1])),
                    'end_time'          : str(UTCDateTime(i[2])),
                    'network'           : [i[3]],
                    'stations_code'     : [i[4]],
                    'classID'           : [i[5]],
                    'class'             : [i[6]]
                    }
                df = pd.DataFrame(data_)
                df.to_csv(directory+'/input/labels_csv/inaseisform.csv', mode='a', header=False, index=False)
                try:
                    if not os.path.exists(directory+'/input/waveform/'+new_fnSZ):
                        new_st.write(directory+'/input/waveform/'+new_fnSZ, format='MSEED')
                        print(f'Success write mseed file for station {i[4]}')
                        print(f'===========================================')
                    else:
                        print(f'file mseed for station {i[4]} is exist')
                        print(f'===========================================')
                # except:
                #     print(f'***!warning!*** >> cant write mseed for station {i[4]}')
                except i.Error as e:
                    print("line: {}, error: {}".format(n.line_num, e))
                except StopIteration:
                    break
        print (f'\n ***finished*** \n')
        # print("\n NdasQ Mumet Mas, dikongkon lopang loping wae ... ora mari mari ...!\n")


    def conditioning_waveform_typetwo(path):
        """
        funtion for conditioning 3 channels, BHZ, BHN, BHE, SHZ, SHN, SHE
        resample to 100Hz for standardization all mseed
        """
        
        if os.path.exists(directory+'/input/labels_csv/inaseisform_typetwo.csv'):
            os.remove(directory+'/input/labels_csv/inaseisform_typetwo.csv')

        if os.path.exists(directory+'/input/labels_csv/inaseisform_typetwo_1.csv'):
            os.remove(directory+'/input/labels_csv/inaseisform_typetwo_1.csv')

        read_csv = pd.read_csv(directory+'/'+path)
        init_stations = read_csv[['files_name','start_time','end_time','network','stations_code','classID','class']]
        arr_init_stations = init_stations.to_numpy()

        df = pd.DataFrame(columns=[
            'id',
            'files_name',
            'start_time',
            'end_time',
            'network',
            'stations_code',
            'channels',
            'classID',
            'class'
        ])
        df.to_csv(directory+'/input/labels_csv/inaseisform_typetwo.csv', mode='a', header=True, index=False)

        df = pd.DataFrame(columns=[
            'id',
            'files_name',
            'start_time',
            'end_time',
            'network',
            'stations_code',
            'channels',
            'classID',
            'class'
        ])
        df.to_csv(directory+'/input/labels_csv/inaseisform_typetwo_1.csv', mode='a', header=True, index=False)


        for n,i in enumerate(tqdm(arr_init_stations, desc='Conditioning/Split Process')):
            st = read(directory+'/input/waveform/'+i[0])
            st_net  = st[0].stats['network']
            st_code = st[0].stats['station']
            st_loc  = st[0].stats['location']
            st_fn   = st_net+'.'+st_code+'.'+st_loc+'.'

            data_ = {
                    'id'                : n,
                    'files_name'        : n,
                    'start_time'        : str(UTCDateTime(i[1])),
                    'end_time'          : str(UTCDateTime(i[2])),
                    'network'           : [i[3]],
                    'stations_code'     : [i[4]],
                    'channels'          : 'BH?',
                    'classID'           : [i[5]],
                    'class'             : [i[6]]
                    }
            df = pd.DataFrame(data_)
            df.to_csv(directory+'/input/labels_csv/inaseisform_typetwo_1.csv', mode='a', header=False, index=False)

            if st.select(id=st_fn+'BHN'):
                new_st = st.select(id=st_fn+'BHN')
                # print(f'resample to 100Hz for station {i[4]}')
                # print(new_st)
                new_fnN = str(n)+'_N'
                data_ = {
                    'id'                : n,
                    'files_name'        : new_fnN,
                    'start_time'        : str(UTCDateTime(i[1])),
                    'end_time'          : str(UTCDateTime(i[2])),
                    'network'           : [i[3]],
                    'stations_code'     : [i[4]],
                    'channels'          : 'BHN',
                    'classID'           : [i[5]],
                    'class'             : [i[6]]
                    }
                df = pd.DataFrame(data_)
                df.to_csv(directory+'/input/labels_csv/inaseisform_typetwo.csv', mode='a', header=False, index=False)
                try:
                    if not os.path.exists(directory+'/input/waveform/'+new_fnN):
                        new_st.write(directory+'/input/waveform/'+new_fnN, format='MSEED')
                        print(f'Success write mseed file for station {i[4]}')
                        print(f'===========================================')
                    else:
                        print(f'file mseed for station {i[4]} is exist')
                        print(f'===========================================')
                except:
                    print(f'***!warning!*** >> cant write mseed for station {i[4]}')


            if st.select(id=st_fn+'BHE'):
                new_st = st.select(id=st_fn+'BHE')
                new_fnE = str(n)+'_E' 
                data_ = {
                    'id'                : n,
                    'files_name'        : new_fnE,
                    'start_time'        : str(UTCDateTime(i[1])),
                    'end_time'          : str(UTCDateTime(i[2])),
                    'network'           : [i[3]],
                    'stations_code'     : [i[4]],
                    'channels'          : 'BHE',
                    'classID'           : [i[5]],
                    'class'             : [i[6]]
                    }
                df = pd.DataFrame(data_)
                df.to_csv(directory+'/input/labels_csv/inaseisform_typetwo.csv', mode='a', header=False, index=False)
                try:
                    if not os.path.exists(directory+'/input/waveform/'+new_fnE):
                        new_st.write(directory+'/input/waveform/'+new_fnE, format='MSEED')
                        print(f'Success write mseed file for station {i[4]}')
                        print(f'===========================================')
                    else:
                        print(f'file mseed for station {i[4]} is exist')
                        print(f'===========================================')
                except:
                    print(f'***!warning!*** >> cant write mseed for station {i[4]}')



            if st.select(id=st_fn+'BHZ'):
                new_st = st.select(id=st_fn+'BHZ')
                new_fnZ = str(n)+'_Z'
                data_ = {
                    'id'                : n,
                    'files_name'        : new_fnZ,
                    'start_time'        : str(UTCDateTime(i[1])),
                    'end_time'          : str(UTCDateTime(i[2])),
                    'network'           : [i[3]],
                    'stations_code'     : [i[4]],
                    'channels'          : 'BHZ',
                    'classID'           : [i[5]],
                    'class'             : [i[6]]
                    }
                df = pd.DataFrame(data_)
                df.to_csv(directory+'/input/labels_csv/inaseisform_typetwo.csv', mode='a', header=False, index=False)
                try:
                    if not os.path.exists(directory+'/input/waveform/'+new_fnZ):
                        new_st.write(directory+'/input/waveform/'+new_fnZ, format='MSEED')
                        print(f'Success write mseed file for station {i[4]}')
                        print(f'===========================================')
                    else:
                        print(f'file mseed for station {i[4]} is exist')
                        print(f'===========================================')
                except:
                    print(f'***!warning!*** >> cant write mseed for station {i[4]}')

                
            if st.select(id=st_fn+'SHN'):
                new_st = st.select(id=st_fn+'SHN')
                new_fnSN = str(n)+'_N'
                data_ = {
                    'id'                : n,
                    'files_name'        : new_fnSN,
                    'start_time'        : str(UTCDateTime(i[1])),
                    'end_time'          : str(UTCDateTime(i[2])),
                    'network'           : [i[3]],
                    'stations_code'     : [i[4]],
                    'channels'          : 'SHN',
                    'classID'           : [i[5]],
                    'class'             : [i[6]]
                    }
                df = pd.DataFrame(data_)
                df.to_csv(directory+'/input/labels_csv/inaseisform_typetwo.csv', mode='a', header=False, index=False)
                try:
                    if not os.path.exists(directory+'/input/waveform/'+new_fnSN):
                        new_st.write(directory+'/input/waveform/'+new_fnSN, format='MSEED')
                        print(f'Success write mseed file for station {i[4]}')
                        print(f'===========================================')
                    else:
                        print(f'file mseed for station {i[4]} is exist')
                        print(f'===========================================')
                except:
                    print(f'***!warning!*** >> cant write mseed for station {i[4]}')

                
            if st.select(id=st_fn+'SHE'):
                new_st = st.select(id=st_fn+'SHE')
                new_fnSE = str(n)+'_E'
                data_ = {
                    'id'                : n,
                    'files_name'        : new_fnSE,
                    'start_time'        : str(UTCDateTime(i[1])),
                    'end_time'          : str(UTCDateTime(i[2])),
                    'network'           : [i[3]],
                    'stations_code'     : [i[4]],
                    'channels'          : 'SHE',
                    'classID'           : [i[5]],
                    'class'             : [i[6]]
                    }
                df = pd.DataFrame(data_)
                df.to_csv(directory+'/input/labels_csv/inaseisform_typetwo.csv', mode='a', header=False, index=False)
                try:
                    if not os.path.exists(directory+'/input/waveform/'+new_fnSE):
                        new_st.write(directory+'/input/waveform/'+new_fnSE, format='MSEED')
                        print(f'Success write mseed file for station {i[4]}')
                        print(f'===========================================')
                    else:
                        print(f'file mseed for station {i[4]} is exist')
                        print(f'===========================================')
                except:
                    print(f'***!warning!*** >> cant write mseed for station {i[4]}')
                

            if st.select(id=st_fn+'SHZ'):
                # new_st = st.select(id=st_fn+'SHZ').resample(100.0)
                new_fnSZ = str(n)+'_Z'

                data_ = {
                    'id'                : n,
                    'files_name'        : new_fnSZ,
                    'start_time'        : str(UTCDateTime(i[1])),
                    'end_time'          : str(UTCDateTime(i[2])),
                    'network'           : [i[3]],
                    'stations_code'     : [i[4]],
                    'channels'          : 'SHZ',
                    'classID'           : [i[5]],
                    'class'             : [i[6]]
                    }
                df = pd.DataFrame(data_)
                df.to_csv(directory+'/input/labels_csv/inaseisform_typetwo.csv', mode='a', header=False, index=False)
                try:
                    if not os.path.exists(directory+'/input/waveform/'+new_fnSZ):
                        new_st.write(directory+'/input/waveform/'+new_fnSZ, format='MSEED')
                        print(f'Success write mseed file for station {i[4]}')
                        print(f'===========================================')
                    else:
                        print(f'file mseed for station {i[4]} is exist')
                        print(f'===========================================')
                # except:
                #     print(f'***!warning!*** >> cant write mseed for station {i[4]}')
                except i.Error as e:
                    print("line: {}, error: {}".format(n.line_num, e))
                except StopIteration:
                    break
        print (f'\n ***finished*** \n')
        # print("\n NdasQ Mumet Mas, dikongkon lopang loping wae ... ora mari mari ...!\n")


    @tf.function
    def _normalize_tensorflow(S, hparams):
        return tf.clip_by_value((S - hparams.min_level_db) / -hparams.min_level_db, 0, 1)

    def _tf_log10(x):
        numerator = tf.math.log(x)
        denominator = tf.math.log(tf.constant(10, dtype=numerator.dtype))
        return numerator / denominator

    def _amp_to_db_tensorflow(x):
        return 20 * Transform._tf_log10(tf.clip_by_value(tf.abs(x), 1e-5, 1e100))

    def _stft_tensorflow(signals, hparams):
        return tf.signal.stft(
            signals,
            hparams.win_length,
            hparams.hop_length,
            hparams.n_fft,
            pad_end=True,
            window_fn=tf.signal.hann_window,
        )

    def spectrogram_tensorflow(y, hparams):
        D = Transform._stft_tensorflow(y, hparams)
        S = Transform._amp_to_db_tensorflow(tf.abs(D)) - hparams.ref_level_db
        return Transform._normalize_tensorflow(S, hparams)


    def make_spectogram(path):
        if not os.path.exists(directory+'/input/spectogram/'):
            os.makedirs(directory+'/input/spectogram')
            print('== folder /input/spectogram/ created')

        read_csv = pd.read_csv(directory+'/'+path)
        init_stations = read_csv[['files_name','start_time','end_time','network','stations_code','classID','class']]
        arr_init_stations = init_stations.to_numpy()

        for i in arr_init_stations:
            st = read(directory+'/input/waveform/'+i[0])
            # print(st.__str__(extended=True))

            data = st[0].data.astype('float32')
            sr = int(st[0].stats.sampling_rate)
            hparams = HParams(  
                # spectrogramming
                win_length = 256,
                n_fft = 256,
                hop_length= 128,
                ref_level_db = 250000,
                min_level_db = -250000,
                # mel scaling
                num_mel_bins = 128,
                mel_lower_edge_hertz = 0,
                mel_upper_edge_hertz = (sr/2),
                # inversion
                power = 1, # for spectral inversion
                griffin_lim_iters = 50,
                pad=True,
                #
            )
            spectrogram = Transform.spectrogram_tensorflow(data.astype('float32'), hparams)
            fig, ax = plt.subplots(ncols=1, figsize=(5,5))
            cax = ax.matshow(spectrogram.numpy().T, aspect='auto', origin='lower')
            # fig.colorbar(cax)
            ax.axis('off')
            fig.savefig(directory+'/input/spectogram/'+i[0]+'.png', bbox_inches='tight',pad_inches = 0, dpi=100)
    

    #be carefull for used ths function, please check again,
    #this function used librosa
    @tf.function
    def make_spectogram_mags(path):
        if not os.path.exists(directory+'/input/dataset_EQ/spectogram/'):
            os.makedirs(directory+'/input/dataset_EQ/spectogram')
            print('== folder /input/spectogram/dataset_EQ/ created')

        read_csv = pd.read_csv(directory+'/'+path)
        init_stations = read_csv[['trace_name','source_magnitude']]
        arr_init_stations = init_stations.to_numpy()

        for i in arr_init_stations:
            if os.path.exists(directory+'/input/dataset_EQ/event/'+i[0]):
                st = read(directory+'/input/dataset_EQ/event/'+i[0])
                # print(st.__str__(extended=True))

                data = st[0].data.astype('float32')
                sr = int(st[0].stats.sampling_rate)
                max_points = int(st[0].stats.npts)
                offset = 0

                hop_length = 128
                n_fft = 256
                cmap = 'jet'
                bins_per_octave = 12
                auto_aspect = False
                y_axis = "linear"  # linear or log
                fmin = None
                fmax = 5.0

                # Librosa spectrogram
                D = librosa.amplitude_to_db(
                    np.abs(librosa.stft(data, hop_length=hop_length, n_fft=n_fft)), ref=np.max)

                fig, ax = plt.subplots()

                img = librosa.display.specshow(D, y_axis=y_axis, sr=sr,
                                               hop_length=hop_length, x_axis='time', ax=ax, cmap=cmap, bins_per_octave=bins_per_octave,
                                               auto_aspect=auto_aspect)

                if fmin is not None:
                    fmin0 = fmin
                else:
                    fmin0 = 0

                if fmax is not None:
                    fmax0 = fmax
                else:
                    fmax0 = sr/2

                ax.set_ylim([fmin, fmax])
                fig.colorbar(img, ax=ax, format="%+2.f dB")
                plt.savefig(directory+'/input/dataset_EQ/spectogram/'+i[0]+'.png', bbox_inches='tight', dpi=300)
                plt.close()

                continue

    def cp_spectogram(path):
        if not os.path.exists(directory+'/input/dataset/'):
            os.makedirs(directory+'/input/dataset')
            print('== folder /input/dataset/ created')

        data_path           = pd.read_csv(path)
        data_path_class     = data_path[['files_name','class']]
        arr_data_path       = data_path_class.to_numpy()
        labels = data_path['class'].value_counts().index.tolist()

        for a,b in enumerate(labels):
            if not os.path.exists(directory+'/input/dataset_spectogram/'+b):
                os.makedirs(directory+'/input/dataset_spectogram/'+b)
                print(f'== folder /input/dataset/{b} created')

            for x in arr_data_path:
                if x[1] == b:
                    src_ = directory+'/input/spectogram/'+x[0]+'.png'
                    des_ = directory+'/input/dataset_spectogram/'+b+'/'
                    if not os.path.exists(directory+'/input/dataset_spectogram/'+b+'/'+x[0]):
                        try:
                            shutil.copy(src_, des_)
                        except shutil.SameFileError:
                            print(f'Source and destination represents the same file.')
                        except IsADirectoryError:
                            print(f'Destination is a directory.')
                        except PermissionError:
                            print(f'Permission denied. check your permision')
                        except:
                            print(f'Error occurred while copying file.')


    def cp_spectogram_mags(path): 
        if not os.path.exists(directory+'/input/dataset_EQ/datasEQ_spectogram/'):
            os.makedirs(directory+'/input/dataset_EQ/datasEQ_spectogram/')
            print('== folder /input/dataset_EQ/datasEQ_spectogram/ created')

        data_path           = pd.read_csv(path)
        data_path_class     = data_path[['trace_name','source_magnitude','receiver_code']]
        arr_data_path       = data_path_class.to_numpy()
        labels = data_path['source_magnitude'].value_counts().index.tolist()

        for a,b in enumerate(labels):
            fold_b = str(b)
            # print(fold_b)
            if not os.path.exists(directory+'/input/dataset_EQ/datasEQ_spectogram/'+fold_b):
                os.makedirs(directory+'/input/dataset_EQ/datasEQ_spectogram/'+fold_b)
                print(f'== folder /input/dataset_EQ/datasEQ_spectogram/{fold_b} created')

            for x in arr_data_path:
                if x[1] == b:
                    src_ = directory+'/input/dataset_EQ/spectogram/'+x[0]+'.png'
                    des_ = directory+'/input/dataset_EQ/datasEQ_spectogram/'+fold_b+'/'
                    if not os.path.exists(directory+'/input/dataset_EQ/datasEQ_spectogram/'+fold_b+'/'+x[0]):
                        try:
                            shutil.copy(src_, des_)
                        except shutil.SameFileError:
                            print(f'Source and destination represents the same file.')
                        except IsADirectoryError:
                            print(f'Destination is a directory.')
                        except PermissionError:
                            print(f'Permission denied. check your permision')
                        except:
                            print(f'Error occurred while copying file. {x[0]} - {x[2]} - please check path files')




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Dec 26 09:31:23 2021

@author: hakimbmkg
"""

import os
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'

import pathlib
import shutil
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import tensorflow as tf
# import h5py
from tensorflow.keras import layers
from tensorflow.keras import models
from IPython import display
from obspy import read
from tqdm import tqdm

seed = 42
tf.random.set_seed(seed)
np.random.seed(seed)

@tf.function
class Datas:
    """
    class for conditioning data, ex: split train and test, shuffle data, and merge to hdf5 file for datase
    """

    global directory
    directory = os.getcwd()

    def split_labels(csv_path):
        """
        read data from csv path and make train and test
        """

        if not os.path.exists(directory+'/input/dataset/'):
            os.makedirs(directory+'/input/dataset')
            print('== folder /input/dataset/ created')

        data_path           = pd.read_csv(csv_path)
        data_path_class     = data_path[['files_name','class']]
        arr_data_path       = data_path_class.to_numpy()
        labels = data_path['class'].value_counts().index.tolist()

        for a,b in enumerate(labels):
            if not os.path.exists(directory+'/input/dataset/'+b):
                os.makedirs(directory+'/input/dataset/'+b)
                print(f'== folder /input/dataset/{b} created')

            for x in arr_data_path:
                if x[1] == b:
                    src_ = directory+'/input/waveform/'+x[0]
                    des_ = directory+'/input/dataset/'+b+'/'
                    if not os.path.exists(directory+'/input/dataset/'+b+'/'+x[0]):
                        try:
                            shutil.copy(src_, des_)
                        except shutil.SameFileError:
                            print(f'Source and destination represents the same file.')
                        except IsADirectoryError:
                            print(f'Destination is a directory.')
                        except PermissionError:
                            print(f'Permission denied. check your permision')
                        except:
                            print(f'Error occurred while copying file.')


    def split_labels_mags(csv_path):
        """
        read data from csv path and make train and test
        """

        if not os.path.exists(directory+'/input/dataset_EQ/datasEQ_spectogram/'):
            os.makedirs(directory+'/input/dataset_EQ/datasEQ_spectogram/')
            print('== folder /input/dataset_EQ/datasEQ_spectogram/ created')

        data_path           = pd.read_csv(csv_path)
        data_path_class     = data_path[['trace_name','source_magnitude','receiver_code']]
        arr_data_path       = data_path_class.to_numpy()
        labels = data_path['source_magnitude'].value_counts().index.tolist()

        for a,b in enumerate(labels):
            fold_b = str(b)
            # print(fold_b)
            if not os.path.exists(directory+'/input/dataset_EQ/datasEQ_spectogram/'+fold_b):
                os.makedirs(directory+'/input/dataset_EQ/datasEQ_spectogram/'+fold_b)
                print(f'== folder /input/dataset_EQ/datasEQ_spectogram/{fold_b} created')

            for x in arr_data_path:
                if x[1] == b:
                    src_ = directory+'/input/dataset_EQ/spectogram/'+x[0]+'.png'
                    des_ = directory+'/input/dataset_EQ/datasEQ_spectogram/'+fold_b+'/'
                    if not os.path.exists(directory+'/input/dataset_EQ/datasEQ_spectogram/'+fold_b+'/'+x[0]):
                        try:
                            shutil.copy(src_, des_)
                        except shutil.SameFileError:
                            print(f'Source and destination represents the same file.')
                        except IsADirectoryError:
                            print(f'Destination is a directory.')
                        except PermissionError:
                            print(f'Permission denied. check your permision')
                        except:
                            print(f'Error occurred while copying file. {x[0]} - {x[2]} - please check path files')


    def shuffle_datas(n_trains, n_vals, n_tests):
        ## reference tensorflow for wav files see tensorflow pages
        data_dir = pathlib.Path(directory+'/input/dataset/')
        commands = np.array(tf.io.gfile.listdir(str(data_dir)))
        filenames = tf.io.gfile.glob(str(data_dir)+'/*/*')
        filenames = tf.random.shuffle(filenames)
        num_samples = len(filenames)

        n_train = int(round((num_samples*int(n_trains))/100))
        n_val   = int(round((num_samples*int(n_vals))/100))
        n_test  = int(round((num_samples*int(n_tests))/100))

        train_files = filenames[:n_train]
        val_files = filenames[n_train: n_train + n_val]
        test_files = filenames[-n_test:]

        print('Training set size', len(train_files))
        print('Validation set size', len(val_files))
        print('Test set size', len(test_files))

        return train_files, val_files, test_files

    def recondition_mseeds(func_):
        print(func_)

    


import os
from setuptools import setup, find_packages
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
	name='linduai',
	version='0.0.1',
	author='hakimBMKG',
	author_email='arif.hakim@bmkg.go.id',
	description='Python Package for Station Quality Analysis and Predicted Magnitude Based on Convolutional Neural Network',
	license='MIT License',
	keywords='Station Earthquake Quality Magnitude Prediction CNN ',
	url='https://github.com/hakimbmkg/LinduAI',
	packages=find_packages(),
	install_requires=[
	'pytest',
	'numpy==1.19.5',
	'pkginfo',
	'scipy==1.7.1', 
	# 'tensorflow==2.4.1', 
	'obspy',
	'librosa'
	], 

    python_requires='>=3.8',
	)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Dec 24 07:06:13 2021

@author: hakimbmkg
"""

import os
import librosa
import librosa.display

from obspy.core.stream import read
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'
import matplotlib.pyplot as plt
import numpy as np
import datetime
import tensorflow as tf

from tensorflow import keras
from tensorflow.keras import layers
from tensorflow.keras.models import Sequential

from pathlib import Path

class Models():
    """
    class for make model and main apps for training and testing
    reference from tensorflow CNN
    """

    def __init__(self) -> None:
        pass

    global directory
    directory = os.getcwd()

    def cnnmodels(num_classes, img_height, img_width):
        model = Sequential([
            tf.keras.layers.experimental.preprocessing.Rescaling(1./255, input_shape=(img_height, img_width, 3)),
            layers.Conv2D(16, 3, padding='same', activation='relu'),
            layers.MaxPooling2D(),
            layers.Conv2D(32, 3, padding='same', activation='relu'),
            layers.MaxPooling2D(),
            layers.Conv2D(64, 3, padding='same', activation='relu'),
            layers.MaxPooling2D(),
            layers.Dropout(0.2),
            layers.Flatten(),
            layers.Dense(128, activation='relu'),
            layers.Dense(num_classes)
            ])
        return(model)

    def trainmodels(path, n_epoch,summary_models = None):
        data_dir = Path(path)
        files_count = len(list(data_dir.glob('*/*.png')))
        print(f'**Summary** files on folder is {files_count} files')

        batch_size = 32
        img_height = 180
        img_width  = 180

        filenames = tf.io.gfile.glob(str(data_dir) + '/*/*')
        filenames = tf.random.shuffle(filenames)
        num_samples = len(filenames)

        train_ds = tf.keras.preprocessing.image_dataset_from_directory(
            data_dir,
            validation_split=0.2,
            subset="training",
            seed=123,
            image_size=(img_height, img_width),
            batch_size=batch_size)

        val_ds = tf.keras.preprocessing.image_dataset_from_directory(
            data_dir,
            validation_split=0.2,
            subset="validation",
            seed=123,
            image_size=(img_height, img_width),
            batch_size=batch_size)

        class_names = train_ds.class_names
        print(f'**Summary** class is {class_names}')

        plt.figure(figsize=(10, 10))
        for images, labels in train_ds.take(1):
            for i in range(9):
                ax = plt.subplot(3, 3, i + 1)
                plt.imshow(images[i].numpy().astype("uint8"))
                plt.title(class_names[labels[i]])
                plt.axis("off")
                plt.savefig(directory+'/input/tmp_files/specvslabels.png', bbox_inches='tight', dpi=300)
      

        for image_batch, labels_batch in train_ds:
            print(f'**Summary** image batch shape {image_batch.shape}')
            print(f'**Summary** labels batch shape{labels_batch.shape}')
            break

        AUTOTUNE = tf.data.AUTOTUNE
        train_ds = train_ds.cache().shuffle(1000).prefetch(buffer_size=AUTOTUNE)
        val_ds = val_ds.cache().prefetch(buffer_size=AUTOTUNE)

        normalization_layer = tf.keras.layers.experimental.preprocessing.Rescaling(1./255, offset=0.0)
        normalized_ds = train_ds.map(lambda x, y: (normalization_layer(x), y))
        image_batch, labels_batch = next(iter(normalized_ds))
        first_image = image_batch[0]
        print(np.min(first_image), np.max(first_image)) 

        num_classes = len(class_names)
        model = Models.cnnmodels(num_classes, img_height, img_width)
        model.compile(optimizer='adam',loss=tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True),metrics=['accuracy'])

        if summary_models == 'summary':
            model.summary()

        epochs = n_epoch
        history = model.fit(train_ds,validation_data=val_ds,epochs=epochs)

        acc = history.history['accuracy']
        val_acc = history.history['val_accuracy']

        loss = history.history['loss']
        val_loss = history.history['val_loss']

        epochs_range = range(epochs)

        filepath = 'input/models_noise/'
        tf.keras.models.save_model(model, filepath)

        plt.figure(figsize=(8, 8))
        plt.subplot(1, 2, 1)
        plt.plot(epochs_range, acc, label='Training Accuracy')
        plt.plot(epochs_range, val_acc, label='Validation Accuracy')
        plt.legend(loc='lower right')
        plt.title('Training and Validation Accuracy')

        plt.subplot(1, 2, 2)
        plt.plot(epochs_range, loss, label='Training Loss')
        plt.plot(epochs_range, val_loss, label='Validation Loss')
        plt.legend(loc='upper right')
        plt.title('Training and Validation Loss')
        # plt.show()
        plt.savefig(directory+'/input/tmp_files/trainvsval.png', bbox_inches='tight', dpi=300)
        plt.close()

        # return history, model

    def predictedstationquality(path, models):
        # files_path = Path(path)
        st          = read(path)
        data        = st[0].data.astype('float32')
        sr          = int(st[0].stats.sampling_rate)
        max_points  = int(st[0].stats.npts)
        Offset      = 0

        hop_length = 128
        n_fft = 256
        cmap = 'jet'
        bins_per_octave = 12
        auto_aspect = False
        y_axis = "linear"  # linear or log
        fmin = None
        fmax = 5.0

        # Librosa spectrogram
        D = librosa.amplitude_to_db(np.abs(librosa.stft(data, hop_length=hop_length, n_fft=n_fft)), ref=np.max)
        fig, ax = plt.subplots()
        img = librosa.display.specshow(D, y_axis=y_axis, sr=sr,
        hop_length=hop_length, x_axis='time', ax=ax, cmap=cmap, bins_per_octave=bins_per_octave,
        auto_aspect=auto_aspect)
        
        if fmin is not None:
            fmin0 = fmin
        else:
            fmin0 = 0

        if fmax is not None:
            fmax0 = fmax
        else:
            fmax0 = sr/2

        ax.set_ylim([fmin, fmax])
        fig.colorbar(img, ax=ax, format="%+2.f dB")
        file_names = str(datetime.datetime.now())+'.png'
        plt.savefig(directory+'/input/tmp_files/'+file_names, bbox_inches='tight', dpi=300)
        plt.close()
        Models.predicted(directory+'/input/tmp_files/'+file_names, models)

    def predicted(path, models):
        class_names = ['Baik','Buruk','Sedang'] 
        model = tf.keras.models.load_model(models)

        files       = path
        files_      = tf.keras.preprocessing.image.load_img(files, target_size=(180,180))
        files_array = tf.keras.preprocessing.image.img_to_array(files_)
        files_array = tf.expand_dims(files_array, 0)

        prediction  = model.predict(files_array)
        score       = tf.nn.softmax(prediction[0])

        print('This index station is =={}== with a == {:.2f}% ==  confidence'.format(class_names[np.argmax(score)], 100 * np.max(score)))


